@@ -2,23 +2,24 @@
 
 from __future__ import annotations
 
-from dataclasses import dataclass
+from dataclasses import dataclass, field
+from typing import Any
 
 import numpy as np
 from loguru import logger
 from numpy.typing import NDArray
 from scipy.spatial.transform import Rotation as Rot
 
-from imu_python.definitions import ACCEL_GRAVITY_MSEC2
+from imu_python.definitions import ACCEL_GRAVITY_MSEC2, FilterConfig, PreConfigStepType
 
 
 @dataclass
 class VectorXYZ:
     """Represent a 3D vector."""
 
-    x: float | NDArray
-    y: float | NDArray
-    z: float | NDArray
+    x: float
+    y: float
+    z: float
 
     @classmethod
     def from_tuple(cls, values: tuple[float, float, float]) -> VectorXYZ:
@@ -51,27 +52,21 @@ class VectorXYZ:
 
     def __repr__(self) -> str:
         """Return a string representation of the object."""
-        if isinstance(self.x, float):
-            return f"VectorXYZ(x={self.x:.3f}, y={self.y:.3f}, z={self.z:.3f})"
-        else:
-            return f"VectorXYZ(x={self.x}, y={self.y}, z={self.z})"
+        return f"VectorXYZ(x={self.x:.3f}, y={self.y:.3f}, z={self.z:.3f})"
 
 
 @dataclass
 class Quaternion:
     """Represent a Quaternion (w, x, y, z)."""
 
-    w: float | NDArray
-    x: float | NDArray
-    y: float | NDArray
-    z: float | NDArray
+    w: float
+    x: float
+    y: float
+    z: float
 
     def __repr__(self) -> str:
         """Return a string representation of the object."""
-        if isinstance(self.x, float):
-            return f"Quaternion(w={self.w:.3f}, x={self.x:.3f}, y={self.y:.3f}, z={self.z:.3f})"
-        else:
-            return f"Quaternion(w={self.w}, x={self.x}, y={self.y}, z={self.z})"
+        return f"Quaternion(w={self.w:.3f}, x={self.x:.3f}, y={self.y:.3f}, z={self.z:.3f})"
 
     def to_euler(self, seq: str) -> VectorXYZ:
         """Convert the Quaternion to an Euler angle (x, y, z)."""
@@ -80,25 +75,96 @@ class Quaternion:
         return VectorXYZ.from_tuple(euler)
 
 
+@dataclass(frozen=True)
+class IMURawData:
+    """Represent raw sensor data."""
+
+    accel: VectorXYZ
+    gyro: VectorXYZ
+    mag: VectorXYZ | None = None
+
+
 @dataclass(frozen=True)
 class IMUData:
     """Represent parsed IMU sensor data."""
 
     timestamp: float
-    accel: VectorXYZ
-    gyro: VectorXYZ
     quat: Quaternion
-    mag: VectorXYZ | None = None
+    raw_data: IMURawData
 
 
 @dataclass
 class IMUConfig:
-    """Configuration data for sensor models."""
+    """Configuration data for sensor models.
+
+    Attributes:
+        name: Name of the IMU.
+        addresses: List of possible I2C addresses.
+        library: Module import path for the driver.
+        module_class: Name of the class inside the module.
+        i2c_param: Name of the I2C parameter in the class constructor.
+        constants_module: Location of the constants/enums (if any) for the PreconfigStep.
+        filter_gain: Gain value for the IMU filter.
+        pre_config: List of pre-configuration steps to initialize/calibrate the IMU.
+
+    """
 
     name: str
     addresses: list[int]
     library: str
-    module_class: str  # name of the class inside the module
+    module_class: str
+    i2c_param: str
+    accel_range_g: float
+    gyro_range_dps: float
+    constants_module: str | None = None
+    filter_gain: float = FilterConfig.gain
+    pre_config: list[PreConfigStep] = field(default_factory=list)
+
+
+@dataclass
+class PreConfigStep:
+    """Class representing a single IMU pre-configuration step.
+
+    Attributes:
+        name: Name of the method or property to configure.
+        args: Positional arguments to pass if it's a callable method.
+        kwargs: Keyword arguments to pass if it's a callable method.
+        step_type: PreConfigStepType.CALL for a method or .SET for a property assignment.
+
+    """
+
+    name: str
+    args: tuple[Any, ...] = ()
+    kwargs: dict[str, Any] = field(default_factory=dict)
+    step_type: PreConfigStepType = PreConfigStepType.CALL
+
+
+@dataclass
+class IMUDataFile:
+    """IMU data reading with Pandas."""
+
+    time: np.ndarray
+    gyros: list[VectorXYZ]
+    accels: list[VectorXYZ]
+    mags: list[VectorXYZ]
+    quats: list[Quaternion]
+
+    def __iter__(self):
+        """Iterate row-by-row, yielding IMUData instances."""
+        n = len(self.time)
+        for i in range(n):
+            imu_data = []
+            data = IMUData(
+                timestamp=float(self.time[i]),
+                quat=self.quats[i],
+                raw_data=IMURawData(
+                    accel=self.accels[i],
+                    gyro=self.gyros[i],
+                    mag=self.mags[i],
+                ),
+            )
+            imu_data.append(data)
+            yield imu_data
 
 
 class AdafruitIMU:
